@@ -1,31 +1,25 @@
 from __future__ import annotations
-from typing import Union, Callable
+from typing import Union
 import math
 import random
-from ..utils import alloneof, almost_equal
-from danielutils import isoneof, validate, NotImplemented
-from ..BaseClasses import Complex____
+from ..utils import isoneof, alloneof, almost_equal
 
 
-class Complex(Complex____):
-
-    __int_float = [[int, float], None, None]
-    __int_float_complex = [[int, float, Complex____], None, None]
-
+class Complex:
     @staticmethod
-    @validate(__int_float, __int_float)
-    def from_polar(r: Union[int, float], theta: Union[int, float]) -> Complex:
+    def fromPolar(r: float, theta: float) -> Complex:
         real = r * math.cos(theta)
         imag = r * math.sin(theta)
         return Complex(real, imag)
 
     @staticmethod
-    @validate(__int_float, __int_float)
-    def from_polar_radians(r: Union[int, float], theta_radians: Union[int, float]) -> Complex:
-        return Complex.from_polar(r, theta_radians*180/math.pi)
+    def fromPolarRadians(r: float, theta_radians: float) -> Complex:
+        return Complex.fromPolar(r, theta_radians*180/math.pi)
 
-    @validate(None, __int_float, __int_float)
-    def __init__(self, real: Union[int, float], imag: Union[int, float]) -> None:
+    def __init__(self, real: float, imag: float) -> None:
+        if not alloneof([real, imag], [int, float]):
+            raise TypeError(
+                "Complex.__init__: real and imag must be int or float")
         self.real = real
         self.imag = imag
 
@@ -60,64 +54,55 @@ class Complex(Complex____):
         elif real != "0" == imag:
             return real
         elif real == "0" != imag:
-            if imag == "1":
-                return "i"
-            elif imag == "-1":
-                return "-i"
             return imag+"i"
         return f'{real} + {imag}i'
 
-    def __repr__(self) -> str:
-        return str(self)
-
-    @validate(None, __int_float_complex)
-    def __add__(self, other: Union[int, float, Complex]) -> Complex:
-        if isoneof(other, [int, float]):
+    def __add__(self, other: Union[float, Complex]) -> Complex:
+        if isinstance(other, float):
             return Complex(self.real + other, self.imag)
-        return Complex(self.real + other.real, self.imag + other.imag)
+        else:
+            return Complex(self.real + other.real, self.imag + other.imag)
 
-    @validate(None, __int_float_complex)
-    def __radd__(self, other: Union[int, float, Complex]) -> Complex:
+    def __radd__(self, other: Union[float, Complex]) -> Complex:
         return self.__add__(other)
 
-    @validate(None, __int_float_complex)
-    def __mul__(self, other: Union[int, float, Complex]) -> Complex:
-        if isoneof(other, [int, float]):
+    def __mul__(self, other: Union[float, Complex]) -> Complex:
+        if isinstance(other, float):
             return Complex(self.real * other, self.imag * other)
-        return Complex(self.real * other.real - self.imag * other.imag,
-                       self.real * other.imag + self.imag * other.real)
+        else:
+            return Complex(self.real * other.real - self.imag * other.imag,
+                           self.real * other.imag + self.imag * other.real)
 
-    @validate(None, __int_float_complex)
-    def __rmul__(self, other: Union[int, float, Complex]) -> Complex:
+    def __rmul__(self, other: Union[float, Complex]) -> Complex:
         return self.__mul__(other)
 
     def __neg__(self) -> Complex:
         return Complex(-self.real, -self.imag)
 
-    @validate(None, __int_float_complex)
-    def __sub__(self, other: Union[int, float, Complex]) -> Complex:
+    def __sub__(self, other: Union[float, Complex]) -> Complex:
         return self + (-other)
 
-    @validate(None, __int_float_complex)
-    def __rsub__(self, other: Union[int, float, Complex]) -> Complex:
+    def __rsub__(self, other: Union[float, Complex]) -> Complex:
         # other-self==-(self-other)
         return -self.__sub__(other)
 
     def __abs__(self) -> float:
         return (self.real ** 2 + self.imag ** 2) ** 0.5
 
-    @validate(None, __int_float_complex)
-    def __eq__(self, other: Union[int, float, Complex]) -> bool:
+    def __eq__(self, other: Complex) -> bool:
+        if not isoneof(other, [int, float, Complex]):
+            raise TypeError(f"cannot compare equality to type {type(other)}")
         if not isinstance(other, Complex):
             other = Complex(other, 0)
         return almost_equal(self.real, other.real) and almost_equal(self.imag, other.imag)
 
-    @validate(None, __int_float_complex)
-    def __ne__(self, other: Union[int, float, Complex]):
+    def __ne__(self, other):
         return not self.__eq__(other)
 
-    @validate(None, __int_float_complex)
     def __truediv__(self, other: Union[int, float, Complex]) -> Complex:
+        if not isinstance(other, Complex) and not isinstance(other, float) and not isinstance(other, int):
+            raise TypeError(
+                "can only divide complex by [int, float , complex]")
         if other == 0:
             raise ZeroDivisionError("Cannot divide by zero")
 
@@ -128,16 +113,19 @@ class Complex(Complex____):
         denominator = other*other.conjugate
         return Complex(nominator.real/denominator.real, nominator.imag/denominator.real)
 
-    @validate(None, __int_float_complex)
-    def __rtruediv__(self, other: Union[int, float, Complex]) -> Complex:
+    def __rtruediv__(self, other) -> Complex:
+        if not isinstance(other, Complex) and not isinstance(other, float) and not isinstance(other, int):
+            raise TypeError(
+                "can only divide [int, float , complex] by complex")
         if not isinstance(other, Complex):
             other = Complex(other, 0)
         nominator = other*self.conjugate
         denominator = self*self.conjugate
         return Complex(nominator.real/denominator.real, nominator.imag/denominator.real)
 
-    @validate(None, int)
-    def __pow__(self, p: int):
+    def __pow__(self, p):
+        if not isoneof(p, [int]):
+            raise TypeError(f"can only raise complex to [int]")
         # TODO look at this again
         # return Complex.fromPolar(self.r**p, self.theta**p)
 
@@ -156,9 +144,9 @@ class Complex(Complex____):
                 res *= self
             return fix_negativ(res)
 
-    @NotImplemented
     def __rpow__(self, other):
-        pass
+        # TODO: implement base**Complex
+        raise NotImplementedError("This is not implemented yet")
 
     @ property
     def conjugate(self):
@@ -169,6 +157,5 @@ class Complex(Complex____):
         return (self * self.conjugate).real
 
     @ staticmethod
-    @validate(__int_float, __int_float, Callable)
     def random(min_val: float = -10, max_val: float = 10, value_func=random.randint) -> Complex:
         return Complex(value_func(min_val, max_val), value_func(min_val, max_val))
